package org.jtheque.primary;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.schema.Schema;
import org.jtheque.core.managers.schema.Insert;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.utils.bean.Version;
import org.jtheque.primary.dao.able.*;

import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;

/**
 * The database schema for the Movies Module.
 *
 * @author Baptiste Wicht
 */
public final class PrimaryUtilsSchema implements Schema {
    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new MoviesSchema.
     */
    public PrimaryUtilsSchema(){
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public Version getVersion() {
        return new Version("1.1");
    }

    @Override
    public String getName() {
        return "PrimaryUtils-Schema";
    }

    @Override
    public void install() {
        createGeneralDataTables();
        createPrimaryDataTables();
        createReferentialIntegrityConstraints();
    }

    @Override
    public void update(Version from) {
        if("1.0".equals(from.getVersion())){
            createReferentialIntegrityConstraints();
            createPrimaryDataTables();
        }
    }

    private void createPrimaryDataTables() {
        jdbcTemplate.update("CREATE TABLE " + IDaoTypes.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(150) NOT NULL, IMPL VARCHAR(20) NOT NULL, CONSTRAINT UNIQUE_TYPE UNIQUE(NAME, IMPL))");
        jdbcTemplate.update("CREATE TABLE " + IDaoKinds.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(150) NOT NULL, IMPL VARCHAR(20) NOT NULL, CONSTRAINT UNIQUE_KIND UNIQUE(NAME, IMPL))");
        jdbcTemplate.update("CREATE TABLE " + IDaoSagas.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(150) NOT NULL, IMPL VARCHAR(20) NOT NULL, CONSTRAINT UNIQUE_SAGA UNIQUE(NAME, IMPL))");
        jdbcTemplate.update("CREATE TABLE " + IDaoLendings.TABLE + " (ID INT IDENTITY PRIMARY KEY, DATE INT NOT NULL, THE_BORROWER_FK INT NOT NULL, IMPL VARCHAR(20) NOT NULL)");
        jdbcTemplate.update("CREATE TABLE " + IDaoCollections.TABLE + " (ID INT IDENTITY PRIMARY KEY, TITLE VARCHAR(150) NOT NULL UNIQUE, PROTECTED BOOLEAN, PASSWORD VARCHAR(150), IMPL VARCHAR(20) NOT NULL)");

        jdbcTemplate.update("CREATE INDEX KINDS_IDX ON " + IDaoKinds.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX SAGAS_IDX ON " + IDaoSagas.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX LENDINGS_IDX ON " + IDaoLendings.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX TYPES_IDX ON " + IDaoTypes.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX COLLECTIONS_IDX ON " + IDaoCollections.TABLE + "(ID)");
    }

    private void createGeneralDataTables() {
        jdbcTemplate.update("CREATE TABLE " + IDaoCountries.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(150) NOT NULL UNIQUE)");
        jdbcTemplate.update("CREATE TABLE " + IDaoLanguages.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(100) NOT NULL UNIQUE)");
        jdbcTemplate.update("CREATE TABLE " + IDaoBorrowers.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(100) NOT NULL, FIRSTNAME VARCHAR(100) NOT NULL, EMAIL VARCHAR(100), THE_COUNTRY_FK INT, CONSTRAINT UNIQUE_BORROWER UNIQUE(NAME, FIRSTNAME))");

        jdbcTemplate.update("CREATE INDEX BORROWERS_IDX ON " + IDaoBorrowers.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX LANGUAGES_IDX ON " + IDaoLanguages.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX COUNTRIES_IDX ON " + IDaoCountries.TABLE + "(ID)");
    }

    private void createReferentialIntegrityConstraints() {
        jdbcTemplate.update("ALTER TABLE " + IDaoBorrowers.TABLE + " ADD FOREIGN KEY (THE_COUNTRY_FK) REFERENCES  " + IDaoCountries.TABLE + "  (ID) ON UPDATE SET NULL");
    }

    @Override
    public void importDataFromHSQL(Iterable<Insert> inserts) {
        for(Insert insert : inserts){
            if("OD_COUNTRY".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoCountries.TABLE + " (ID, NAME) VALUES(?, ?)", insert.getInt(0), insert.getString(2));
            } else if("OD_LANGUAGE".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoLanguages.TABLE + " (ID, NAME) VALUES(?, ?)", insert.getInt(0), insert.getString(2));
            } else if("OD_BORROWER".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoBorrowers.TABLE + " (ID, NAME, FIRSTNAME, EMAIL, THE_COUNTRY_FK) VALUES(?,?,?,?,?)", insert.getInt(0), insert.getString(3),
                        insert.getString(2), insert.getString(4), insert.getInt(5));
            }
        }
    }
}