package org.jtheque.primary.controller.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.view.able.components.IModel;
import org.jtheque.core.managers.view.able.components.TabComponent;
import org.jtheque.core.managers.view.able.controller.AbstractController;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.primary.controller.able.ControllerState;
import org.jtheque.primary.od.abstraction.Data;

import java.util.List;

/**
 * An abstract principal Controller.
 *
 * @author Baptiste Wicht
 */
public abstract class PrincipalController extends AbstractController {
    private ControllerState state;
    private ControllerState viewState;
    private ControllerState modifyState;
    private ControllerState newObjectState;
    private ControllerState autoAddState;

    @Override
    public final void displayView() {
        Managers.getManager(IViewManager.class).getViews().setSelectedView((TabComponent) getView());

        super.displayView();
    }

    /**
     * Return the state state. This state is active when we show a data on the view.
     *
     * @return a new state to set or <code>null</code> if we doesn't must change state.
     */
    public final ControllerState getViewState() {
        return viewState;
    }

    /**
     * Return the auto add state. This state is active when we add automatically a data.
     *
     * @return a new state to set or <code>null</code> if we doesn't must change state.
     */
    public final ControllerState getAutoAddState() {
        return autoAddState;
    }

    /**
     * Return the modify state. This state is active when we modify a data.
     *
     * @return a new state to set or <code>null</code> if we doesn't must change state.
     */
    public final ControllerState getModifyState() {
        return modifyState;
    }

    /**
     * Return the new object state. This state is active when we add a new data.
     *
     * @return a new state to set or <code>null</code> if we doesn't must change state.
     */
    public final ControllerState getNewObjectState() {
        return newObjectState;
    }

    /**
     * Return the state of the controller.
     *
     * @return The state of the controller.
     */
    public final ControllerState getState() {
        return state;
    }

    /**
     * Set the state of the controller.
     *
     * @param state The new state to apply.
     */
    public final void setAndApplyState(ControllerState state) {
        this.state = state;

        state.apply();
    }

    /**
     * Set the current state. 
     * 
     * @param state The state to set. 
     */
    public void setState(ControllerState state) {
        this.state = state;
    }

    /**
     * Set the view state of the controller.
     *
     * @param viewState The view state.
     */
    public final void setViewState(ControllerState viewState) {
        this.viewState = viewState;
    }

    /**
     * Set the modify object state of the controller.
     *
     * @param modifyState The modify object state.
     */
    public final void setModifyState(ControllerState modifyState) {
        this.modifyState = modifyState;
    }

    /**
     * Set the new object state of the controller.
     *
     * @param newObjectState The new object state.
     */
    public final void setNewObjectState(ControllerState newObjectState) {
        this.newObjectState = newObjectState;
    }

    /**
     * Set the auto add state of the controller.
     *
     * @param autoAddState The auto add state.
     */
    public final void setAutoAddState(ControllerState autoAddState) {
        this.autoAddState = autoAddState;
    }

    /**
     * Return the model of the view managed by the controller.
     *
     * @return The model of the view.
     */
    public abstract IModel getViewModel();

    /**
     * Return the associated data type. 
     * 
     * @return The associated data type. 
     */
    public abstract String getDataType();

    /**
     * Return the display list. 
     * 
     * @return The display list. 
     */
    public abstract List<? extends Data> getDisplayList();
}