package org.jtheque.primary.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.primary.dao.able.IDaoSagas;
import org.jtheque.primary.od.SagaImpl;
import org.jtheque.primary.od.TypeImpl;
import org.jtheque.primary.od.abstraction.Saga;
import org.jtheque.primary.PrimaryUtils;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.utils.StringUtils;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.ArrayList;

/**
 * Cette classe permet de grer la persistance des genres.
 *
 * @author Baptiste Wicht
 */
public final class DaoSagas extends GenericDao<SagaImpl> implements IDaoSagas {
    private final ParameterizedRowMapper<SagaImpl> rowMapper = new SagaRowMapper();
    private final QueryMapper queryMapper = new SagaQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new DaoSagas.
     */
    public DaoSagas() {
        super(TABLE);
    }

    @Override
    public void create(SagaImpl entity) {
        entity.setPrimaryImpl(PrimaryUtils.getPrimaryImpl());
        
        super.create(entity);
    }

    @Override
    public List<SagaImpl> getSagas() {
        return getSagas(PrimaryUtils.getPrimaryImpl());
    }

    private List<SagaImpl> getSagas(String impl) {
        if (StringUtils.isEmpty(impl)) {
            return getAll();
        }

        load();

        List<SagaImpl> sagas = new ArrayList<SagaImpl>(getCache().size() / 2);

        for (SagaImpl saga : getCache().values()) {
            if (impl.equals(saga.getPrimaryImpl())) {
                sagas.add(saga);
            }
        }

        return sagas;
    }

    @Override
    public SagaImpl getSaga(int id) {
        return get(id);
    }

    @Override
    protected ParameterizedRowMapper<SagaImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<SagaImpl> sagas = persistenceContext.getSortedList(TABLE, rowMapper);

        for (SagaImpl saga : sagas) {
            getCache().put(saga.getId(), saga);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        SagaImpl editor = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, editor);
    }

    /**
     * A mapper to map a resultset to a saga.
     *
     * @author Baptiste Wicht
     */
    private static final class SagaRowMapper implements ParameterizedRowMapper<SagaImpl> {
        @Override
        public SagaImpl mapRow(ResultSet rs, int i) throws SQLException {
            SagaImpl saga = new SagaImpl();

            saga.setId(rs.getInt("ID"));
            saga.setName(rs.getString("NAME"));
            saga.setPrimaryImpl(rs.getString("IMPL"));

            return saga;
        }
    }

    /**
     * A mapper to map a saga to an SQL query.
     *
     * @author Baptiste Wicht
     */
    private static final class SagaQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            Saga saga = (Saga) entity;

            String query = "INSERT INTO " + TABLE + " (NAME, IMPL) VALUES(?, ?)";

            Object[] parameters = {
                    saga.getName(), 
                    saga.getPrimaryImpl()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            Saga saga = (Saga) entity;

            String query = "UPDATE " + TABLE + " SET NAME = ?, IMPL = ? WHERE ID = ?";

            Object[] parameters = {
                    saga.getName(),
                    saga.getPrimaryImpl(),
                    saga.getId()};

            return new Query(query, parameters);
        }
    }
}