package org.jtheque.primary.view.impl.frames;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.managers.view.impl.actions.JThequeAction;
import org.jtheque.core.utils.ui.ValidationUtils;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.core.managers.view.impl.frame.abstraction.SwingDialogView;
import org.jtheque.utils.ui.SwingUtils;
import org.jtheque.utils.ui.GridBagUtils;
import org.jtheque.primary.od.abstraction.Data;
import org.jtheque.primary.od.abstraction.Borrower;
import org.jtheque.primary.view.able.IBorrowerView;
import org.jtheque.primary.view.impl.models.BorrowerModel;

import javax.swing.JTextField;
import javax.annotation.PostConstruct;

import java.awt.Container;
import java.awt.Frame;
import java.util.List;

/**
 * Cette classe est l'interface permettant d'ajouter ou de modifier un Emprunteur.
 *
 * @author Baptiste Wicht
 */
public final class BorrowerView extends SwingDialogView implements IBorrowerView {
    private static final long serialVersionUID = -962666643153770662L;

    /* Graphics components */
    private JTextField fieldNom;
    private JTextField fieldPrenom;
    private JTextField fieldEmail;

    private JThequeAction validateAction;
    private JThequeAction closeAction;
    private static final int DEFAULT_COLUMNS = 15;
    private static final int DEFAULT_FIELD_LENGTH = 50;

    /**
     * Constructeur utilis pour ajouter un nouveau genre.
     * 
     * @param frame The parent frame
     */
    public BorrowerView(Frame frame) {
        super(frame);
    }

    @Override
    public void reload() {
        setTitleKey("borrower.view.title");

        fieldNom.setText("");
        fieldPrenom.setText("");
        fieldEmail.setText("");

        getModel().setBorrower(null);
    }

    @Override
    public void reload(Data data) {
        Borrower borrower = (Borrower) data;

        setTitle(getMessage("borrower.view.title.edit") + borrower.getName());

        fieldNom.setText(borrower.getName());
        fieldPrenom.setText(borrower.getFirstName());
        fieldEmail.setText(borrower.getEmail());

        getModel().setBorrower(borrower);
    }

    @Override
    @PostConstruct
    public void build() {
        setContentPane(buildContentPane());
        setResizable(true);
        pack();

        SwingUtils.centerFrame(this);
        
        reload();
    }

    /**
     * Cette mthode initialise et retourne le contentPane de la fentre.
     *
     * @return Le contentPane
     */
    private Container buildContentPane() {
        PanelBuilder builder = new PanelBuilder();
        
        builder.addI18nLabel("borrower.view.name", builder.gbcSet(0, 0));
        builder.addI18nLabel("borrower.view.firstname", builder.gbcSet(0, 1));
        builder.addI18nLabel("borrower.view.mail", builder.gbcSet(0, 2));
        
        fieldNom = builder.add(new JTextField(DEFAULT_COLUMNS), builder.gbcSet(1, 0));
        SwingUtils.addFieldValidateAction(fieldNom, validateAction);
        SwingUtils.addFieldLenghtLimit(fieldNom, DEFAULT_FIELD_LENGTH);
        
        fieldPrenom = builder.add(new JTextField(DEFAULT_COLUMNS), builder.gbcSet(1, 1));
        SwingUtils.addFieldValidateAction(fieldPrenom, validateAction);
        SwingUtils.addFieldLenghtLimit(fieldPrenom, DEFAULT_FIELD_LENGTH);

        fieldEmail = builder.add(new JTextField(DEFAULT_COLUMNS), builder.gbcSet(1, 2));
        SwingUtils.addFieldValidateAction(fieldEmail, validateAction);
        SwingUtils.addFieldLenghtLimit(fieldEmail, DEFAULT_FIELD_LENGTH);
        
        builder.addButtonBar(builder.gbcSet(0, 3, GridBagUtils.NONE, GridBagUtils.BASELINE_LEADING, 2, 1), validateAction, closeAction);
        
        return builder.getPanel();
    }

    @Override
    public JTextField getFieldNom() {
        return fieldNom;
    }

    @Override
    public JTextField getFieldPrenom() {
        return fieldPrenom;
    }

    @Override
    public JTextField getFieldEmail() {
        return fieldEmail;
    }

    @Override
    public void refreshText() {
        super.refreshText();

        if (getModel().getBorrower() != null) {
            setTitle(getMessage("borrower.view.title.edit") + getModel().getBorrower().getName());
        }
    }

    @Override
    public BorrowerModel getModel() {
        return (BorrowerModel) super.getModel();
    }

    @Override
    protected void validate(List<JThequeError> errors) {
        ValidationUtils.rejectIfEmpty(fieldNom.getText(), "borrower.view.name", errors);
        ValidationUtils.rejectIfEmpty(fieldPrenom.getText(), "borrower.view.firstname", errors);

        ValidationUtils.rejectIfLongerThan(fieldNom.getText(), "borrower.view.name", 100, errors);
        ValidationUtils.rejectIfLongerThan(fieldPrenom.getText(), "borrower.view.firstname", 100, errors);
    }

    /**
     * Set the action to launch the validate the view. This is not for use, this only for Spring Injection. 
     * 
     * @param validateAction The action. 
     */
    public void setValidateAction(JThequeAction validateAction) {
        this.validateAction = validateAction;
    }

    /**
     * Set the action to launch the close the view. This is not for use, this only for Spring Injection. 
     * 
     * @param closeAction The action. 
     */
    public void setCloseAction(JThequeAction closeAction) {
        this.closeAction = closeAction;
    }
}