package org.jtheque.primary.view.impl.models.tree;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.ArrayList;
import java.util.List;

/**
 * Represents the root element of a tree model.
 *
 * @author Baptiste Wicht
 */
public final class RootElement {
    private final List<Category> categories;
    private final List<Element> elements;
    private final String name;

    /**
     * Constructs a new <code>RootElement</code>.
     *
     * @param name The name of element
     */
    public RootElement(String name) {
        super();

        this.name = name;
        categories = new ArrayList<Category>(10);
        elements = new ArrayList<Element>(25);
    }

    /**
     * Add a new category to the element.
     *
     * @param category The new category
     */
    public void addCategory(Category category) {
        categories.add(category);
    }

    /**
     * Remove all the categories of the root element.
     */
    public void removeAllCategories() {
        categories.clear();
    }

    /**
     * Indicate if the root element has category or not.
     *
     * @return <code>true</code> if he has one or more categories else <code>false</code>
     */
    public boolean hasCategory() {
        return !categories.isEmpty();
    }

    /**
     * Return the category at the specified position.
     *
     * @param index The position of the category
     * @return The category
     */
    public Category getCategory(int index) {
        return categories.get(index);
    }

    /**
     * Return the element at the specified position.
     *
     * @param index The position of the element
     * @return The element
     */
    public Element getElement(int index) {
        return elements.get(index);
    }

    /**
     * Return the position of a child.
     *
     * @param child The child to search
     * @return The position of the child
     */
    public int getIndexOf(Object child) {
        if (child instanceof Category) {
            return getIndexOf((Category) child);
        }

        return getIndexOf((Element) child);
    }

    /**
     * Return the position of a element.
     *
     * @param element The element to search.
     * @return The position of the element
     */
    private int getIndexOf(Element element) {
        return elements.indexOf(element);
    }

    /**
     * Return the position of a category.
     *
     * @param category The category to search
     * @return The position of the category
     */
    private int getIndexOf(Category category) {
        return categories.indexOf(category);
    }

    /**
     * Add a new element directly to the root element.
     *
     * @param element The new element
     */
    public void addElement(Element element) {
        elements.add(element);
    }

    /**
     * Remove all the elements of the root element.
     */
    public void removeAllElements() {
        elements.clear();
    }

    /**
     * Return the number of elements of the category.
     *
     * @return The sum of the elements on the category
     */
    public int getElementsCount() {
        if (hasCategory()) {
            return categories.size();
        }

        return elements.size();
    }

    @Override
    public String toString() {
        return name;
    }
}